@@ -1,6 +1,6 @@
 // file      : xsde/cxx/parser/expat/document.cxx
 // author    : Boris Kolpackov <boris@codesynthesis.com>
-// copyright : Copyright (c) 2005-2008 Code Synthesis Tools CC
+// copyright : Copyright (c) 2005-2009 Code Synthesis Tools CC
 // license   : GNU GPL v2 + exceptions; see accompanying LICENSE file
 
 #include <xsde/cxx/config.hxx>
@@ -257,10 +257,23 @@ namespace xsde
           {
             ~stream_exception_controller ()
             {
-              if (is_.fail () && is_.eof ())
-                is_.clear (is_.rdstate () & ~std::istream::failbit);
+              std::istream::iostate s = is_.rdstate ();
+              s &= ~std::istream::failbit;
 
-              is_.exceptions (old_state_);
+              // If our error state (sans failbit) intersects with the
+              // exception state then that means we have an active
+              // exception and changing error/exception state will
+              // cause another to be thrown.
+              //
+              if (!(old_state_ & s))
+              {
+                // Clear failbit if it was caused by eof.
+                //
+                if (is_.fail () && is_.eof ())
+                  is_.clear (s);
+
+                is_.exceptions (old_state_);
+              }
             }
 
             stream_exception_controller (std::istream& is)
@@ -990,16 +1003,21 @@ namespace xsde
           //
           if (polymorphic_)
           {
-#if defined (XSDE_STL) || defined (XSDE_EXCEPTIONS)
+#if defined (XSDE_STL)
             prefixes_.push_back (p ? p : "");
             prefix_namespaces_.push_back (ns ? ns : "");
 #else
-            if (prefixes_.push_back (p ? p : "" ) ||
-                prefix_namespaces_.push_back (ns ? ns : ""))
+#if defined (XSDE_EXCEPTIONS)
+            prefixes_.push_back_copy (p ? p : "");
+            prefix_namespaces_.push_back_copy (ns ? ns : "");
+#else
+            if (prefixes_.push_back_copy (p ? p : "" ) ||
+                prefix_namespaces_.push_back_copy (ns ? ns : ""))
             {
               context_.sys_error (sys_error::no_memory);
               XML_StopParser (xml_parser_, false);
             }
+#endif
 #endif
           }
         }
