@@ -1,6 +1,6 @@
 // file      : xsde/cxx/qname.cxx
 // author    : Boris Kolpackov <boris@codesynthesis.com>
-// copyright : Copyright (c) 2005-2008 Code Synthesis Tools CC
+// copyright : Copyright (c) 2005-2009 Code Synthesis Tools CC
 // license   : GNU GPL v2 + exceptions; see accompanying LICENSE file
 
 #include <string.h> // strlen, memcpy
@@ -17,7 +17,7 @@ namespace xsde
 #else
     void qname::
 #endif
-    prefix (const char* prefix)
+    prefix_copy (const char* prefix)
     {
       delete[] prefix_;
 
@@ -30,7 +30,6 @@ namespace xsde
         if (prefix_ == 0)
           return error_no_memory;
 #endif
-
         memcpy (prefix_, prefix, size);
       }
       else
@@ -46,7 +45,7 @@ namespace xsde
 #else
     void qname::
 #endif
-    name (const char* name)
+    name_copy (const char* name)
     {
       delete[] name_;
 
@@ -59,7 +58,6 @@ namespace xsde
         if (name_ == 0)
           return error_no_memory;
 #endif
-
         memcpy (name_, name, size);
       }
       else
